@@ -50,6 +50,12 @@ struct SharedMediaMergedKey {
 
 };
 
+rpl::producer<SparseIdsMergedSlice> SharedScheduledMediaViewer(
+	not_null<Main::Session*> session,
+	SharedMediaMergedKey key,
+	int limitBefore,
+	int limitAfter);
+
 rpl::producer<SparseIdsMergedSlice> SharedMediaMergedViewer(
 	not_null<Main::Session*> session,
 	SharedMediaMergedKey key,
@@ -60,9 +66,9 @@ class SharedMediaWithLastSlice {
 public:
 	using Type = Storage::SharedMediaType;
 
-	using Value = base::variant<FullMsgId, not_null<PhotoData*>>;
+	using Value = std::variant<FullMsgId, not_null<PhotoData*>>;
 	using MessageId = SparseIdsMergedSlice::UniversalMsgId;
-	using UniversalMsgId = base::variant<
+	using UniversalMsgId = std::variant<
 		MessageId,
 		not_null<PhotoData*>>;
 
@@ -71,13 +77,14 @@ public:
 			PeerId peerId,
 			PeerId migratedPeerId,
 			Type type,
-			UniversalMsgId universalId)
+			UniversalMsgId universalId,
+			bool scheduled = false)
 		: peerId(peerId)
 		, migratedPeerId(migratedPeerId)
 		, type(type)
-		, universalId(universalId) {
-			Expects(base::get_if<MessageId>(&universalId) != nullptr
-				|| type == Type::ChatPhoto);
+		, universalId(universalId)
+		, scheduled(scheduled) {
+			Expects(v::is<MessageId>(universalId) || type == Type::ChatPhoto);
 		}
 
 		bool operator==(const Key &other) const {
@@ -94,6 +101,7 @@ public:
 		PeerId migratedPeerId = 0;
 		Type type = Type::kCount;
 		UniversalMsgId universalId;
+		bool scheduled = false;
 
 	};
 
@@ -120,8 +128,8 @@ public:
 		return {
 			key.peerId,
 			key.migratedPeerId,
-			base::get_if<MessageId>(&key.universalId)
-				? (*base::get_if<MessageId>(&key.universalId))
+			v::is<MessageId>(key.universalId)
+				? v::get<MessageId>(key.universalId)
 				: ServerMaxMsgId - 1
 		};
 	}
@@ -135,7 +143,7 @@ public:
 
 private:
 	static std::optional<SparseIdsMergedSlice> EndingSlice(const Key &key) {
-		return base::get_if<MessageId>(&key.universalId)
+		return v::is<MessageId>(key.universalId)
 			? base::make_optional(SparseIdsMergedSlice(EndingKey(key)))
 			: std::nullopt;
 	}
@@ -156,17 +164,15 @@ private:
 		return (a && b) ? base::make_optional(*a + *b) : std::nullopt;
 	}
 	static Value ComputeId(PeerId peerId, MsgId msgId) {
-		return FullMsgId(
-			peerIsChannel(peerId) ? peerToBareInt(peerId) : 0,
-			msgId);
+		return FullMsgId(peerToChannel(peerId), msgId);
 	}
 	static Value ComputeId(const Key &key) {
-		if (auto messageId = base::get_if<MessageId>(&key.universalId)) {
+		if (const auto messageId = std::get_if<MessageId>(&key.universalId)) {
 			return (*messageId >= 0)
 				? ComputeId(key.peerId, *messageId)
 				: ComputeId(key.migratedPeerId, ServerMaxMsgId + *messageId);
 		}
-		return *base::get_if<not_null<PhotoData*>>(&key.universalId);
+		return v::get<not_null<PhotoData*>>(key.universalId);
 	}
 
 	bool isolatedInSlice() const {
